# Instance Types for EC2.  These types are not provided by the AWS SDK, and so
#   are enumerated here manually from the following sources:
#
#   http://aws.amazon.com/ec2/instance-types
#   http://docs.amazonwebservices.com/AWSEC2/latest/UserGuide/instance-types.html
#   http://aws.amazon.com/ec2/previous-generation
#   http://www.ec2instances.info/
#   http://docs.aws.amazon.com/AWSEC2/latest/UserGuide/t2-instances.html
#   http://docs.aws.amazon.com/AWSEC2/latest/UserGuide/c4-instances.html
#     NOTE: We may want to consider using the source data directly, however
#           there aren't the discontinued types.
#           https://raw.githubusercontent.com/powdahound/ec2instances.info/master/www/instances.json
module ManageIQ::Providers::Amazon::InstanceTypes
  # Types that are currently advertised for use
  AVAILABLE_TYPES = {
    "t2.nano"     => {
      :name                    => "t2.nano",
      :family                  => "General Purpose",
      :description             => "T2 Nano",
      :memory                  => 0.5.gigabyte,
      :vcpu                    => 1,
      :ebs_only                => true,
      :instance_store_size     => 0,
      :instance_store_volumes  => 0,
      :architecture            => [:i386, :x86_64],
      :virtualization_type     => [:hvm],
      :network_performance     => :low,
      :physical_processor      => "Intel Xeon Family",
      :processor_clock_speed   => 3.3, # GHz
      :intel_aes_ni            => true,
      :intel_avx               => true,
      :intel_avx2              => nil,
      :intel_turbo             => true,
      :ebs_optimized_available => nil,
      :enhanced_networking     => nil,
      :cluster_networking      => nil,
      :vpc_only                => true,
    },

    "t2.micro"    => {
      :default                 => true,
      :name                    => "t2.micro",
      :family                  => "General Purpose",
      :description             => "T2 Micro",
      :memory                  => 1.gigabyte,
      :vcpu                    => 1,
      :ebs_only                => true,
      :instance_store_size     => 0,
      :instance_store_volumes  => 0,
      :architecture            => [:i386, :x86_64],
      :virtualization_type     => [:hvm],
      :network_performance     => :low_to_moderate,
      :physical_processor      => "Intel Xeon Family",
      :processor_clock_speed   => 3.3, # GHz
      :intel_aes_ni            => true,
      :intel_avx               => true,
      :intel_avx2              => nil,
      :intel_turbo             => true,
      :ebs_optimized_available => nil,
      :enhanced_networking     => nil,
      :cluster_networking      => nil,
      :vpc_only                => true,
    },

    "t2.small"    => {
      :name                    => "t2.small",
      :family                  => "General Purpose",
      :description             => "T2 Small",
      :memory                  => 2.gigabyte,
      :vcpu                    => 1,
      :ebs_only                => true,
      :instance_store_size     => 0,
      :instance_store_volumes  => 0,
      :architecture            => [:i386, :x86_64],
      :virtualization_type     => [:hvm],
      :network_performance     => :low_to_moderate,
      :physical_processor      => "Intel Xeon Family",
      :processor_clock_speed   => 3.3, # GHz
      :intel_aes_ni            => true,
      :intel_avx               => true,
      :intel_avx2              => nil,
      :intel_turbo             => true,
      :ebs_optimized_available => nil,
      :enhanced_networking     => nil,
      :cluster_networking      => nil,
      :vpc_only                => true,
    },

    "t2.medium"   => {
      :name                    => "t2.medium",
      :family                  => "General Purpose",
      :description             => "T2 Medium",
      :memory                  => 4.gigabyte,
      :vcpu                    => 2,
      :ebs_only                => true,
      :instance_store_size     => 0,
      :instance_store_volumes  => 0,
      :architecture            => [:x86_64],
      :virtualization_type     => [:hvm],
      :network_performance     => :low_to_moderate,
      :physical_processor      => "Intel Xeon Family",
      :processor_clock_speed   => 3.3, # GHz
      :intel_aes_ni            => true,
      :intel_avx               => true,
      :intel_avx2              => nil,
      :intel_turbo             => true,
      :ebs_optimized_available => nil,
      :enhanced_networking     => nil,
      :cluster_networking      => nil,
      :vpc_only                => true,
    },

    "t2.large"    => {
      :name                    => "t2.large",
      :family                  => "General Purpose",
      :description             => "T2 Large",
      :memory                  => 8.gigabyte,
      :vcpu                    => 2,
      :ebs_only                => true,
      :instance_store_size     => 0,
      :instance_store_volumes  => 0,
      :architecture            => [:x86_64],
      :virtualization_type     => [:hvm],
      :network_performance     => :low_to_moderate,
      :physical_processor      => "Intel Xeon Family",
      :processor_clock_speed   => 3.0, # GHz
      :intel_aes_ni            => true,
      :intel_avx               => true,
      :intel_avx2              => nil,
      :intel_turbo             => true,
      :ebs_optimized_available => nil,
      :enhanced_networking     => nil,
      :cluster_networking      => nil,
      :vpc_only                => true,
    },

    "m3.medium"   => {
      :name                    => "m3.medium",
      :family                  => "General Purpose",
      :description             => "M3 General Purpose Medium",
      :memory                  => 3.75.gigabytes,
      :vcpu                    => 1,
      :ebs_only                => false,
      :instance_store_size     => 4.gigabytes,
      :instance_store_volumes  => 1, # SSD
      :architecture            => [:x86_64],
      :virtualization_type     => [:paravirtual, :hvm],
      :network_performance     => :moderate,
      :physical_processor      => "Intel Xeon E5-2670v2",
      :processor_clock_speed   => 2.5, # GHz
      :intel_aes_ni            => true,
      :intel_avx               => true,
      :intel_avx2              => nil,
      :intel_turbo             => true,
      :ebs_optimized_available => nil,
      :enhanced_networking     => nil,
      :cluster_networking      => nil,
      :vpc_only                => false,
    },

    "m3.large"    => {
      :name                    => "m3.large",
      :family                  => "General Purpose",
      :description             => "M3 General Purpose Large",
      :memory                  => 7.5.gigabytes,
      :vcpu                    => 2,
      :ebs_only                => false,
      :instance_store_size     => 32.gigabytes,
      :instance_store_volumes  => 1, # SSD
      :architecture            => [:x86_64],
      :virtualization_type     => [:paravirtual, :hvm],
      :network_performance     => :moderate,
      :physical_processor      => "Intel Xeon E5-2670v2",
      :processor_clock_speed   => 2.5, # GHz
      :intel_aes_ni            => true,
      :intel_avx               => true,
      :intel_avx2              => nil,
      :intel_turbo             => true,
      :ebs_optimized_available => nil,
      :enhanced_networking     => nil,
      :cluster_networking      => nil,
      :vpc_only                => false,
    },

    "m3.xlarge"   => {
      :name                    => "m3.xlarge",
      :family                  => "General Purpose",
      :description             => "M3 General Purpose Extra Large",
      :memory                  => 15.gigabytes,
      :vcpu                    => 4,
      :ebs_only                => false,
      :instance_store_size     => 80.gigabytes,
      :instance_store_volumes  => 2, # SSD
      :architecture            => [:x86_64],
      :virtualization_type     => [:paravirtual, :hvm],
      :network_performance     => :high,
      :physical_processor      => "Intel Xeon E5-2670v2",
      :processor_clock_speed   => 2.5, # GHz
      :intel_aes_ni            => true,
      :intel_avx               => true,
      :intel_avx2              => nil,
      :intel_turbo             => true,
      :ebs_optimized_available => true,
      :enhanced_networking     => nil,
      :cluster_networking      => nil,
      :vpc_only                => false,
    },

    "m3.2xlarge"  => {
      :name                    => "m3.2xlarge",
      :family                  => "General Purpose",
      :description             => "M3 General Purpose Double Extra Large",
      :memory                  => 30.gigabytes,
      :vcpu                    => 8,
      :ebs_only                => false,
      :instance_store_size     => 160.gigabytes,
      :instance_store_volumes  => 2, # SSD
      :architecture            => [:x86_64],
      :virtualization_type     => [:paravirtual, :hvm],
      :network_performance     => :high,
      :physical_processor      => "Intel Xeon E5-2670v2",
      :processor_clock_speed   => 2.5, # GHz
      :intel_aes_ni            => true,
      :intel_avx               => true,
      :intel_avx2              => nil,
      :intel_turbo             => true,
      :ebs_optimized_available => true,
      :enhanced_networking     => nil,
      :cluster_networking      => nil,
      :vpc_only                => false,
    },

    "m4.large"    => {
      :name                    => "m4.large",
      :family                  => "General Purpose",
      :description             => "M4 General Purpose Large",
      :memory                  => 8.gigabytes,
      :vcpu                    => 2,
      :ebs_only                => true,
      :instance_store_size     => 0,
      :instance_store_volumes  => 0,
      :architecture            => [:x86_64],
      :virtualization_type     => [:hvm],
      :network_performance     => :moderate,
      :physical_processor      => "Intel Xeon E5-2676v3",
      :processor_clock_speed   => 2.4, # GHz
      :intel_aes_ni            => true,
      :intel_avx               => true,
      :intel_avx2              => true,
      :intel_turbo             => true,
      :ebs_optimized_available => true,
      :enhanced_networking     => true,
      :cluster_networking      => true,
      :vpc_only                => true,
    },

    "m4.xlarge"   => {
      :name                    => "m4.xlarge",
      :family                  => "General Purpose",
      :description             => "M4 General Purpose Extra Large",
      :memory                  => 16.gigabytes,
      :vcpu                    => 4,
      :ebs_only                => true,
      :instance_store_size     => 0,
      :instance_store_volumes  => 0,
      :architecture            => [:x86_64],
      :virtualization_type     => [:hvm],
      :network_performance     => :high,
      :physical_processor      => "Intel Xeon E5-2676v3",
      :processor_clock_speed   => 2.4, # GHz
      :intel_aes_ni            => true,
      :intel_avx               => true,
      :intel_avx2              => true,
      :intel_turbo             => true,
      :ebs_optimized_available => true,
      :enhanced_networking     => true,
      :cluster_networking      => true,
      :vpc_only                => true,
    },

    "m4.2xlarge"  => {
      :name                    => "m4.2xlarge",
      :family                  => "General Purpose",
      :description             => "M4 General Purpose Double Extra Large",
      :memory                  => 32.gigabytes,
      :vcpu                    => 8,
      :ebs_only                => true,
      :instance_store_size     => 0,
      :instance_store_volumes  => 0,
      :architecture            => [:x86_64],
      :virtualization_type     => [:hvm],
      :network_performance     => :high,
      :physical_processor      => "Intel Xeon E5-2676v3",
      :processor_clock_speed   => 2.4, # GHz
      :intel_aes_ni            => true,
      :intel_avx               => true,
      :intel_avx2              => true,
      :intel_turbo             => true,
      :ebs_optimized_available => true,
      :enhanced_networking     => true,
      :cluster_networking      => true,
      :vpc_only                => true,
    },

    "m4.4xlarge"  => {
      :name                    => "m4.4xlarge",
      :family                  => "General Purpose",
      :description             => "M4 General Purpose Quadruple Extra Large",
      :memory                  => 64.gigabytes,
      :vcpu                    => 16,
      :ebs_only                => true,
      :instance_store_size     => 0,
      :instance_store_volumes  => 0,
      :architecture            => [:x86_64],
      :virtualization_type     => [:hvm],
      :network_performance     => :high,
      :physical_processor      => "Intel Xeon E5-2676v3",
      :processor_clock_speed   => 2.4, # GHz
      :intel_aes_ni            => true,
      :intel_avx               => true,
      :intel_avx2              => true,
      :intel_turbo             => true,
      :ebs_optimized_available => true,
      :enhanced_networking     => true,
      :cluster_networking      => true,
      :vpc_only                => true,
    },

    "m4.10xlarge" => {
      :name                    => "m4.10xlarge",
      :family                  => "General Purpose",
      :description             => "M4 General Purpose Ten Extra Large",
      :memory                  => 160.gigabytes,
      :vcpu                    => 40,
      :ebs_only                => true,
      :instance_store_size     => 0,
      :instance_store_volumes  => 0,
      :architecture            => [:x86_64],
      :virtualization_type     => [:hvm],
      :network_performance     => :very_high,
      :physical_processor      => "Intel Xeon E5-2676v3",
      :processor_clock_speed   => 2.4, # GHz
      :intel_aes_ni            => true,
      :intel_avx               => true,
      :intel_avx2              => true,
      :intel_turbo             => true,
      :ebs_optimized_available => true,
      :enhanced_networking     => true,
      :cluster_networking      => true,
      :vpc_only                => true,
    },

    "c3.large"    => {
      :name                    => "c3.large",
      :family                  => "Compute Optimized",
      :description             => "C3 High-CPU Large",
      :memory                  => 3.75.gigabytes,
      :vcpu                    => 2,
      :ebs_only                => false,
      :instance_store_size     => 32.gigabytes,
      :instance_store_volumes  => 2, # SSD
      :architecture            => [:x86_64],
      :virtualization_type     => [:paravirtual, :hvm],
      :network_performance     => :moderate,
      :physical_processor      => "Intel Xeon E5-2680v2",
      :processor_clock_speed   => 2.8, # GHz
      :intel_aes_ni            => true,
      :intel_avx               => true,
      :intel_avx2              => nil,
      :intel_turbo             => true,
      :ebs_optimized_available => nil,
      :enhanced_networking     => true,
      :cluster_networking      => true,
      :vpc_only                => false,
    },

    "c3.xlarge"   => {
      :name                    => "c3.xlarge",
      :family                  => "Compute Optimized",
      :description             => "C3 High-CPU Extra Large",
      :memory                  => 7.5.gigabytes,
      :vcpu                    => 4,
      :ebs_only                => false,
      :instance_store_size     => 80.gigabytes,
      :instance_store_volumes  => 2, # SSD
      :architecture            => [:x86_64],
      :virtualization_type     => [:paravirtual, :hvm],
      :network_performance     => :moderate,
      :physical_processor      => "Intel Xeon E5-2680v2",
      :processor_clock_speed   => 2.8, # GHz
      :intel_aes_ni            => true,
      :intel_avx               => true,
      :intel_avx2              => nil,
      :intel_turbo             => true,
      :ebs_optimized_available => true,
      :enhanced_networking     => true,
      :cluster_networking      => true,
      :vpc_only                => false,
    },

    "c3.2xlarge"  => {
      :name                    => "c3.2xlarge",
      :family                  => "Compute Optimized",
      :description             => "C3 High-CPU Double Extra Large",
      :memory                  => 15.gigabytes,
      :vcpu                    => 8,
      :ebs_only                => false,
      :instance_store_size     => 160.gigabytes,
      :instance_store_volumes  => 2, # SSD
      :architecture            => [:x86_64],
      :virtualization_type     => [:paravirtual, :hvm],
      :network_performance     => :high,
      :physical_processor      => "Intel Xeon E5-2680v2",
      :processor_clock_speed   => 2.8, # GHz
      :intel_aes_ni            => true,
      :intel_avx               => true,
      :intel_avx2              => nil,
      :intel_turbo             => true,
      :ebs_optimized_available => true,
      :enhanced_networking     => true,
      :cluster_networking      => true,
      :vpc_only                => false,
    },

    "c3.4xlarge"  => {
      :name                    => "c3.4xlarge",
      :family                  => "Compute Optimized",
      :description             => "C3 High-CPU Quadruple Extra Large",
      :memory                  => 30.gigabytes,
      :vcpu                    => 16,
      :ebs_only                => false,
      :instance_store_size     => 320.gigabytes,
      :instance_store_volumes  => 2, # SSD
      :architecture            => [:x86_64],
      :virtualization_type     => [:paravirtual, :hvm],
      :network_performance     => :high,
      :physical_processor      => "Intel Xeon E5-2680v2",
      :processor_clock_speed   => 2.8, # GHz
      :intel_aes_ni            => true,
      :intel_avx               => true,
      :intel_avx2              => nil,
      :intel_turbo             => true,
      :ebs_optimized_available => true,
      :enhanced_networking     => true,
      :cluster_networking      => true,
      :vpc_only                => false,
    },

    "c3.8xlarge"  => {
      :name                    => "c3.8xlarge",
      :family                  => "Compute Optimized",
      :description             => "C3 High-CPU Eight Extra Large",
      :memory                  => 60.gigabytes,
      :vcpu                    => 32,
      :ebs_only                => false,
      :instance_store_size     => 640.gigabytes,
      :instance_store_volumes  => 2, # SSD
      :architecture            => [:x86_64],
      :virtualization_type     => [:paravirtual, :hvm],
      :network_performance     => :very_high,
      :physical_processor      => "Intel Xeon E5-2680v2",
      :processor_clock_speed   => 2.8, # GHz
      :intel_aes_ni            => true,
      :intel_avx               => true,
      :intel_avx2              => nil,
      :intel_turbo             => true,
      :ebs_optimized_available => nil,
      :enhanced_networking     => true,
      :cluster_networking      => true,
      :vpc_only                => false,
    },

    "c4.large"    => {
      :name                    => "c4.large",
      :family                  => "Compute Optimized",
      :description             => "C4 High-CPU Large",
      :memory                  => 3.75.gigabytes,
      :vcpu                    => 2,
      :ebs_only                => true,
      :instance_store_size     => 0,
      :instance_store_volumes  => 0,
      :architecture            => [:x86_64],
      :virtualization_type     => [:hvm],
      :network_performance     => :moderate,
      :physical_processor      => "Intel Xeon E5-2666v3",
      :processor_clock_speed   => 2.9, # GHz
      :intel_aes_ni            => true,
      :intel_avx               => true,
      :intel_avx2              => true,
      :intel_turbo             => true,
      :ebs_optimized_available => true,
      :enhanced_networking     => true,
      :cluster_networking      => true,
      :vpc_only                => true,
    },

    "c4.xlarge"   => {
      :name                    => "c4.xlarge",
      :family                  => "Compute Optimized",
      :description             => "C4 High-CPU Extra Large",
      :memory                  => 7.5.gigabytes,
      :vcpu                    => 4,
      :ebs_only                => true,
      :instance_store_size     => 0,
      :instance_store_volumes  => 0,
      :architecture            => [:x86_64],
      :virtualization_type     => [:hvm],
      :network_performance     => :moderate,
      :physical_processor      => "Intel Xeon E5-2666v3",
      :processor_clock_speed   => 2.9, # GHz
      :intel_aes_ni            => true,
      :intel_avx               => true,
      :intel_avx2              => true,
      :intel_turbo             => true,
      :ebs_optimized_available => true,
      :enhanced_networking     => true,
      :cluster_networking      => true,
      :vpc_only                => true,
    },

    "c4.2xlarge"  => {
      :name                    => "c4.2xlarge",
      :family                  => "Compute Optimized",
      :description             => "C4 High-CPU Double Extra Large",
      :memory                  => 15.gigabytes,
      :vcpu                    => 8,
      :ebs_only                => true,
      :instance_store_size     => 0,
      :instance_store_volumes  => 0,
      :architecture            => [:x86_64],
      :virtualization_type     => [:hvm],
      :network_performance     => :high,
      :physical_processor      => "Intel Xeon E5-2666v3",
      :processor_clock_speed   => 2.9, # GHz
      :intel_aes_ni            => true,
      :intel_avx               => true,
      :intel_avx2              => true,
      :intel_turbo             => true,
      :ebs_optimized_available => true,
      :enhanced_networking     => true,
      :cluster_networking      => true,
      :vpc_only                => true,
    },

    "c4.4xlarge"  => {
      :name                    => "c4.4xlarge",
      :family                  => "Compute Optimized",
      :description             => "C4 High-CPU Quadruple Extra Large",
      :memory                  => 30.gigabytes,
      :vcpu                    => 16,
      :ebs_only                => true,
      :instance_store_size     => 0,
      :instance_store_volumes  => 0,
      :architecture            => [:x86_64],
      :virtualization_type     => [:hvm],
      :network_performance     => :high,
      :physical_processor      => "Intel Xeon E5-2666v3",
      :processor_clock_speed   => 2.9, # GHz
      :intel_aes_ni            => true,
      :intel_avx               => true,
      :intel_avx2              => true,
      :intel_turbo             => true,
      :ebs_optimized_available => true,
      :enhanced_networking     => true,
      :cluster_networking      => true,
      :vpc_only                => true,
    },

    "c4.8xlarge"  => {
      :name                    => "c4.8xlarge",
      :family                  => "Compute Optimized",
      :description             => "C4 High-CPU Eight Extra Large",
      :memory                  => 60.gigabytes,
      :vcpu                    => 36,
      :ebs_only                => true,
      :instance_store_size     => 0,
      :instance_store_volumes  => 0,
      :architecture            => [:x86_64],
      :virtualization_type     => [:hvm],
      :network_performance     => :very_high,
      :physical_processor      => "Intel Xeon E5-2666v3",
      :processor_clock_speed   => 2.9, # GHz
      :intel_aes_ni            => true,
      :intel_avx               => true,
      :intel_avx2              => true,
      :intel_turbo             => true,
      :ebs_optimized_available => true,
      :enhanced_networking     => true,
      :cluster_networking      => true,
      :vpc_only                => true,
    },

    "g2.2xlarge"  => {
      :name                    => "g2.2xlarge",
      :family                  => "GPU Instances",
      :description             => "G2 Double Extra Large",
      :memory                  => 15.gigabytes,
      :vcpu                    => 8,
      :ebs_only                => false,
      :instance_store_size     => 60.gigabytes,
      :instance_store_volumes  => 1, # SSD
      :architecture            => [:x86_64],
      :virtualization_type     => [:hvm],
      :network_performance     => :high,
      :physical_processor      => "Intel Xeon E5-2670",
      :processor_clock_speed   => 2.6, # GHz
      :intel_aes_ni            => true,
      :intel_avx               => true,
      :intel_avx2              => nil,
      :intel_turbo             => true,
      :ebs_optimized_available => true,
      :enhanced_networking     => nil,
      :cluster_networking      => true,
      :vpc_only                => false,
    },

    "g2.8xlarge"  => {
      :name                    => "g2.8xlarge",
      :family                  => "GPU Instances",
      :description             => "G2 Eight Extra Large",
      :memory                  => 60.gigabytes,
      :vcpu                    => 32,
      :ebs_only                => false,
      :instance_store_size     => 240.gigabytes,
      :instance_store_volumes  => 2, # SSD
      :architecture            => [:x86_64],
      :virtualization_type     => [:hvm],
      :network_performance     => :very_high,
      :physical_processor      => "Intel Xeon E5-2670",
      :processor_clock_speed   => 2.6, # GHz
      :intel_aes_ni            => true,
      :intel_avx               => true,
      :intel_avx2              => nil,
      :intel_turbo             => true,
      :ebs_optimized_available => nil,
      :enhanced_networking     => nil,
      :cluster_networking      => true,
      :vpc_only                => false,
    },

    "r3.large"    => {
      :name                    => "r3.large",
      :family                  => "Memory Optimized",
      :description             => "R3 High-Memory Large",
      :memory                  => 15.gigabytes,
      :vcpu                    => 2,
      :ebs_only                => false,
      :instance_store_size     => 32.gigabytes,
      :instance_store_volumes  => 1, # SSD
      :architecture            => [:x86_64],
      :virtualization_type     => [:hvm],
      :network_performance     => :moderate,
      :physical_processor      => "Intel Xeon E5-2670v2",
      :processor_clock_speed   => 2.5, # GHz
      :intel_aes_ni            => true,
      :intel_avx               => true,
      :intel_avx2              => nil,
      :intel_turbo             => true,
      :ebs_optimized_available => nil,
      :enhanced_networking     => true,
      :cluster_networking      => nil,
      :vpc_only                => false,
    },

    "r3.xlarge"   => {
      :name                    => "r3.xlarge",
      :family                  => "Memory Optimized",
      :description             => "R3 High-Memory Extra Large",
      :memory                  => 30.5.gigabytes,
      :vcpu                    => 4,
      :ebs_only                => false,
      :instance_store_size     => 80.gigabytes,
      :instance_store_volumes  => 1, # SSD
      :architecture            => [:x86_64],
      :virtualization_type     => [:hvm],
      :network_performance     => :moderate,
      :physical_processor      => "Intel Xeon E5-2670v2",
      :processor_clock_speed   => 2.5, # GHz
      :intel_aes_ni            => true,
      :intel_avx               => true,
      :intel_avx2              => nil,
      :intel_turbo             => true,
      :ebs_optimized_available => true,
      :enhanced_networking     => true,
      :cluster_networking      => nil,
      :vpc_only                => false,
    },

    "r3.2xlarge"  => {
      :name                    => "r3.2xlarge",
      :family                  => "Memory Optimized",
      :description             => "R3 High-Memory Double Extra Large",
      :memory                  => 61.gigabytes,
      :vcpu                    => 8,
      :ebs_only                => false,
      :instance_store_size     => 160.gigabytes,
      :instance_store_volumes  => 1, # SSD
      :architecture            => [:x86_64],
      :virtualization_type     => [:hvm],
      :network_performance     => :high,
      :physical_processor      => "Intel Xeon E5-2670v2",
      :processor_clock_speed   => 2.5, # GHz
      :intel_aes_ni            => true,
      :intel_avx               => true,
      :intel_avx2              => nil,
      :intel_turbo             => true,
      :ebs_optimized_available => true,
      :enhanced_networking     => true,
      :cluster_networking      => nil,
      :vpc_only                => false,
    },

    "r3.4xlarge"  => {
      :name                    => "r3.4xlarge",
      :family                  => "Memory Optimized",
      :description             => "R3 High-Memory Quadruple Extra Large",
      :memory                  => 122.gigabytes,
      :vcpu                    => 16,
      :ebs_only                => false,
      :instance_store_size     => 320.gigabytes,
      :instance_store_volumes  => 1, # SSD
      :architecture            => [:x86_64],
      :virtualization_type     => [:hvm],
      :network_performance     => :high,
      :physical_processor      => "Intel Xeon E5-2670v2",
      :processor_clock_speed   => 2.5, # GHz
      :intel_aes_ni            => true,
      :intel_avx               => true,
      :intel_avx2              => nil,
      :intel_turbo             => true,
      :ebs_optimized_available => true,
      :enhanced_networking     => true,
      :cluster_networking      => nil,
      :vpc_only                => false,
    },

    "r3.8xlarge"  => {
      :name                    => "r3.8xlarge",
      :family                  => "Memory Optimized",
      :description             => "R3 High-Memory Eight Extra Large",
      :memory                  => 244.gigabytes,
      :vcpu                    => 32,
      :ebs_only                => false,
      :instance_store_size     => 640.gigabytes,
      :instance_store_volumes  => 2, # SSD
      :architecture            => [:x86_64],
      :virtualization_type     => [:hvm],
      :network_performance     => :very_high,
      :physical_processor      => "Intel Xeon E5-2670v2",
      :processor_clock_speed   => 2.5, # GHz
      :intel_aes_ni            => true,
      :intel_avx               => true,
      :intel_avx2              => nil,
      :intel_turbo             => true,
      :ebs_optimized_available => nil,
      :enhanced_networking     => true,
      :cluster_networking      => nil,
      :vpc_only                => false,
    },

    "i2.xlarge"   => {
      :name                    => "i2.xlarge",
      :family                  => "Storage Optimized",
      :description             => "I2 Extra Large",
      :memory                  => 30.5.gigabytes,
      :vcpu                    => 4,
      :ebs_only                => false,
      :instance_store_size     => 800.gigabytes,
      :instance_store_volumes  => 1, # SSD
      :architecture            => [:x86_64],
      :virtualization_type     => [:hvm],
      :network_performance     => :moderate,
      :physical_processor      => "Intel Xeon E5-2670v2",
      :processor_clock_speed   => 2.5, # GHz
      :intel_aes_ni            => true,
      :intel_avx               => true,
      :intel_avx2              => nil,
      :intel_turbo             => true,
      :ebs_optimized_available => true,
      :enhanced_networking     => true,
      :cluster_networking      => true,
      :vpc_only                => false,
    },

    "i2.2xlarge"  => {
      :name                    => "i2.2xlarge",
      :family                  => "Storage Optimized",
      :description             => "I2 Double Extra Large",
      :memory                  => 61.gigabytes,
      :vcpu                    => 8,
      :ebs_only                => false,
      :instance_store_size     => 1_600.gigabytes,
      :instance_store_volumes  => 2, # SSD
      :architecture            => [:x86_64],
      :virtualization_type     => [:hvm],
      :network_performance     => :high,
      :physical_processor      => "Intel Xeon E5-2670v2",
      :processor_clock_speed   => 2.5, # GHz
      :intel_aes_ni            => true,
      :intel_avx               => true,
      :intel_avx2              => nil,
      :intel_turbo             => true,
      :ebs_optimized_available => true,
      :enhanced_networking     => true,
      :cluster_networking      => true,
      :vpc_only                => false,
    },

    "i2.4xlarge"  => {
      :name                    => "i2.4xlarge",
      :family                  => "Storage Optimized",
      :description             => "I2 Quadruple Extra Large",
      :memory                  => 122.gigabytes,
      :vcpu                    => 16,
      :ebs_only                => false,
      :instance_store_size     => 3_200.gigabytes,
      :instance_store_volumes  => 4, # SSD
      :architecture            => [:x86_64],
      :virtualization_type     => [:hvm],
      :network_performance     => :high,
      :physical_processor      => "Intel Xeon E5-2670v2",
      :processor_clock_speed   => 2.5, # GHz
      :intel_aes_ni            => true,
      :intel_avx               => true,
      :intel_avx2              => nil,
      :intel_turbo             => true,
      :ebs_optimized_available => true,
      :enhanced_networking     => true,
      :cluster_networking      => true,
      :vpc_only                => false,
    },

    "i2.8xlarge"  => {
      :name                    => "i2.8xlarge",
      :family                  => "Storage Optimized",
      :description             => "I2 Eight Extra Large",
      :memory                  => 244.gigabytes,
      :vcpu                    => 32,
      :ebs_only                => false,
      :instance_store_size     => 6_400.gigabytes,
      :instance_store_volumes  => 8, # SSD
      :architecture            => [:x86_64],
      :virtualization_type     => [:hvm],
      :network_performance     => :very_high,
      :physical_processor      => "Intel Xeon E5-2670v2",
      :processor_clock_speed   => 2.5, # GHz
      :intel_aes_ni            => true,
      :intel_avx               => true,
      :intel_avx2              => nil,
      :intel_turbo             => true,
      :ebs_optimized_available => nil,
      :enhanced_networking     => true,
      :cluster_networking      => true,
      :vpc_only                => false,
    },

    "d2.xlarge"   => {
      :name                    => "d2.xlarge",
      :family                  => "Storage Optimized",
      :description             => "D2 Extra Large",
      :memory                  => 30.5.gigabytes,
      :vcpu                    => 4,
      :ebs_only                => false,
      :instance_store_size     => 6_000.gigabytes,
      :instance_store_volumes  => 3,
      :architecture            => [:x86_64],
      :virtualization_type     => [:hvm],
      :network_performance     => :moderate,
      :physical_processor      => "Intel Xeon E5-2676v3",
      :processor_clock_speed   => 2.4, # GHz
      :intel_aes_ni            => true,
      :intel_avx               => true,
      :intel_avx2              => true,
      :intel_turbo             => true,
      :ebs_optimized_available => true,
      :enhanced_networking     => true,
      :cluster_networking      => true,
      :vpc_only                => false,
    },

    "d2.2xlarge"  => {
      :name                    => "d2.2xlarge",
      :family                  => "Storage Optimized",
      :description             => "D2 Double Extra Large",
      :memory                  => 61.gigabytes,
      :vcpu                    => 8,
      :ebs_only                => false,
      :instance_store_size     => 12_000.gigabytes,
      :instance_store_volumes  => 6,
      :architecture            => [:x86_64],
      :virtualization_type     => [:hvm],
      :network_performance     => :high,
      :physical_processor      => "Intel Xeon E5-2676v3",
      :processor_clock_speed   => 2.4, # GHz
      :intel_aes_ni            => true,
      :intel_avx               => true,
      :intel_avx2              => true,
      :intel_turbo             => true,
      :ebs_optimized_available => true,
      :enhanced_networking     => true,
      :cluster_networking      => true,
      :vpc_only                => false,
    },

    "d2.4xlarge"  => {
      :name                    => "d2.4xlarge",
      :family                  => "Storage Optimized",
      :description             => "D2 Quadruple Extra Large",
      :memory                  => 122.gigabytes,
      :vcpu                    => 16,
      :ebs_only                => false,
      :instance_store_size     => 24_000.gigabytes,
      :instance_store_volumes  => 12,
      :architecture            => [:x86_64],
      :virtualization_type     => [:hvm],
      :network_performance     => :high,
      :physical_processor      => "Intel Xeon E5-2676v3",
      :processor_clock_speed   => 2.4, # GHz
      :intel_aes_ni            => true,
      :intel_avx               => true,
      :intel_avx2              => true,
      :intel_turbo             => true,
      :ebs_optimized_available => true,
      :enhanced_networking     => true,
      :cluster_networking      => true,
      :vpc_only                => false,
    },

    "d2.8xlarge"  => {
      :name                    => "d2.8xlarge",
      :family                  => "Storage Optimized",
      :description             => "D2 Eight Extra Large",
      :memory                  => 244.gigabytes,
      :vcpu                    => 36,
      :ebs_only                => false,
      :instance_store_size     => 48_000.gigabytes,
      :instance_store_volumes  => 24,
      :architecture            => [:x86_64],
      :virtualization_type     => [:hvm],
      :network_performance     => :very_high,
      :physical_processor      => "Intel Xeon E5-2676v3",
      :processor_clock_speed   => 2.4, # GHz
      :intel_aes_ni            => true,
      :intel_avx               => true,
      :intel_avx2              => true,
      :intel_turbo             => true,
      :ebs_optimized_available => true,
      :enhanced_networking     => true,
      :cluster_networking      => true,
      :vpc_only                => false,
    },

    "x1.32xlarge"  => {
      :name                    => "x1.32xlarge",
      :family                  => "Memory Optimized",
      :description             => "X1 32xlarge",
      :memory                  => 1952.gigabytes,
      :vcpu                    => 128,
      :ebs_only                => false,
      :instance_store_size     => 1920.gigabytes,
      :instance_store_volumes  => 2,
      :architecture            => [:x86_64],
      :virtualization_type     => [:hvm],
      :network_performance     => :very_high,
      :physical_processor      => "Intel Xeon E7-8880 v3",
      :processor_clock_speed   => 2.3, # GHz
      :intel_aes_ni            => true,
      :intel_avx               => true,
      :intel_avx2              => true,
      :intel_turbo             => true,
      :ebs_optimized_available => true,
      :enhanced_networking     => true,
      :cluster_networking      => true,
      :vpc_only                => false,
    },
  }

  # Types that are still advertised, but not recommended for new instances.
  DEPRECATED_TYPES = {
    "t1.micro"    => {
      :name                    => "t1.micro",
      :family                  => "Micro Instances",
      :description             => "T1 Micro",
      :memory                  => 613.megabytes,
      :vcpu                    => 1,
      :ebs_only                => true,
      :instance_store_size     => 0,
      :instance_store_volumes  => 0,
      :architecture            => [:i386, :x86_64],
      :virtualization_type     => [:paravirtual],
      :network_performance     => :very_low,
      :physical_processor      => "Variable",
      :processor_clock_speed   => nil,
      :intel_aes_ni            => nil,
      :intel_avx               => nil,
      :intel_avx2              => nil,
      :intel_turbo             => nil,
      :ebs_optimized_available => nil,
      :enhanced_networking     => nil,
      :cluster_networking      => nil,
      :vpc_only                => false,
    },

    "m1.small"    => {
      :name                    => "m1.small",
      :family                  => "General Purpose",
      :description             => "M1 General Purpose Small",
      :memory                  => 1.7.gigabytes,
      :vcpu                    => 1,
      :ebs_only                => false,
      :instance_store_size     => 160.gigabytes,
      :instance_store_volumes  => 1,
      :architecture            => [:i386, :x86_64],
      :virtualization_type     => [:paravirtual],
      :network_performance     => :low,
      :physical_processor      => "Intel Xeon Family",
      :processor_clock_speed   => nil,
      :intel_aes_ni            => nil,
      :intel_avx               => nil,
      :intel_avx2              => nil,
      :intel_turbo             => nil,
      :ebs_optimized_available => nil,
      :enhanced_networking     => nil,
      :cluster_networking      => nil,
      :vpc_only                => false,
    },

    "m1.medium"   => {
      :name                    => "m1.medium",
      :family                  => "General Purpose",
      :description             => "M1 General Purpose Medium",
      :memory                  => 3.75.gigabytes,
      :vcpu                    => 1,
      :ebs_only                => false,
      :instance_store_size     => 410.gigabytes,
      :instance_store_volumes  => 1,
      :architecture            => [:i386, :x86_64],
      :virtualization_type     => [:paravirtual],
      :network_performance     => :moderate,
      :physical_processor      => "Intel Xeon Family",
      :processor_clock_speed   => nil,
      :intel_aes_ni            => nil,
      :intel_avx               => nil,
      :intel_avx2              => nil,
      :intel_turbo             => nil,
      :ebs_optimized_available => nil,
      :enhanced_networking     => nil,
      :cluster_networking      => nil,
      :vpc_only                => false,
    },

    "m1.large"    => {
      :name                    => "m1.large",
      :family                  => "General Purpose",
      :description             => "M1 General Purpose Large",
      :memory                  => 7.5.gigabytes,
      :vcpu                    => 2,
      :ebs_only                => false,
      :instance_store_size     => 840.gigabytes,
      :instance_store_volumes  => 2,
      :architecture            => [:x86_64],
      :virtualization_type     => [:paravirtual],
      :network_performance     => :moderate,
      :physical_processor      => "Intel Xeon Family",
      :processor_clock_speed   => nil,
      :intel_aes_ni            => nil,
      :intel_avx               => nil,
      :intel_avx2              => nil,
      :intel_turbo             => nil,
      :ebs_optimized_available => true,
      :enhanced_networking     => nil,
      :cluster_networking      => nil,
      :vpc_only                => false,
    },

    "m1.xlarge"   => {
      :name                    => "m1.xlarge",
      :family                  => "General Purpose",
      :description             => "M1 General Purpose Extra Large",
      :memory                  => 15.gigabytes,
      :vcpu                    => 4,
      :ebs_only                => false,
      :instance_store_size     => 1680.gigabytes,
      :instance_store_volumes  => 4,
      :architecture            => [:x86_64],
      :virtualization_type     => [:paravirtual],
      :network_performance     => :high,
      :physical_processor      => "Intel Xeon Family",
      :processor_clock_speed   => nil,
      :intel_aes_ni            => nil,
      :intel_avx               => nil,
      :intel_avx2              => nil,
      :intel_turbo             => nil,
      :ebs_optimized_available => true,
      :enhanced_networking     => nil,
      :cluster_networking      => nil,
      :vpc_only                => false,
    },

    "c1.medium"   => {
      :name                    => "c1.medium",
      :family                  => "Compute Optimized",
      :description             => "C1 High-CPU Medium",
      :memory                  => 1.7.gigabytes,
      :vcpu                    => 2,
      :ebs_only                => false,
      :instance_store_size     => 350.gigabytes,
      :instance_store_volumes  => 1,
      :architecture            => [:i386, :x86_64],
      :virtualization_type     => [:paravirtual],
      :network_performance     => :moderate,
      :physical_processor      => "Intel Xeon Family",
      :processor_clock_speed   => nil,
      :intel_aes_ni            => true,
      :intel_avx               => nil,
      :intel_avx2              => nil,
      :intel_turbo             => nil,
      :ebs_optimized_available => nil,
      :enhanced_networking     => nil,
      :cluster_networking      => nil,
      :vpc_only                => false,
    },

    "c1.xlarge"   => {
      :name                    => "c1.xlarge",
      :family                  => "Compute Optimized",
      :description             => "C1 High-CPU Extra Large",
      :memory                  => 7.gigabytes,
      :vcpu                    => 8,
      :ebs_only                => false,
      :instance_store_size     => 1680.gigabytes,
      :instance_store_volumes  => 4,
      :architecture            => [:x86_64],
      :virtualization_type     => [:paravirtual],
      :network_performance     => :high,
      :physical_processor      => "Intel Xeon Family",
      :processor_clock_speed   => nil,
      :intel_aes_ni            => true,
      :intel_avx               => nil,
      :intel_avx2              => nil,
      :intel_turbo             => nil,
      :ebs_optimized_available => true,
      :enhanced_networking     => nil,
      :cluster_networking      => nil,
      :vpc_only                => false,
    },

    "cc2.8xlarge" => {
      :name                    => "cc2.8xlarge",
      :family                  => "Compute Optimized",
      :description             => "Cluster Compute Eight Extra Large",
      :memory                  => 60.5.gigabytes,
      :vcpu                    => 32,
      :ebs_only                => false,
      :instance_store_size     => 3360.gigabytes,
      :instance_store_volumes  => 4,
      :architecture            => [:x86_64],
      :virtualization_type     => [:hvm],
      :network_performance     => :very_high,
      :physical_processor      => "Intel Xeon E5-2670",
      :processor_clock_speed   => 2.6, # GHz
      :intel_aes_ni            => true,
      :intel_avx               => true,
      :intel_avx2              => nil,
      :intel_turbo             => true,
      :ebs_optimized_available => nil,
      :enhanced_networking     => nil,
      :cluster_networking      => nil,
      :vpc_only                => false,
    },

    "cg1.4xlarge" => {
      :name                    => "cg1.4xlarge",
      :family                  => "GPU Instances",
      :description             => "Cluster GPU Quadruple Extra Large",
      :memory                  => 22.5.gigabytes,
      :vcpu                    => 16,
      :ebs_only                => false,
      :instance_store_size     => 1680.gigabytes,
      :instance_store_volumes  => 2,
      :architecture            => [:x86_64],
      :virtualization_type     => [:hvm],
      :network_performance     => :very_high,
      :physical_processor      => "Intel Xeon x5570",
      :processor_clock_speed   => nil,
      :intel_aes_ni            => true,
      :intel_avx               => nil,
      :intel_avx2              => nil,
      :intel_turbo             => nil,
      :ebs_optimized_available => nil,
      :enhanced_networking     => nil,
      :cluster_networking      => nil,
      :vpc_only                => false,
    },

    "m2.xlarge"   => {
      :name                    => "m2.xlarge",
      :family                  => "Memory Optimized",
      :description             => "M2 High-Memory Extra Large",
      :memory                  => 17.1.gigabytes,
      :vcpu                    => 2,
      :ebs_only                => false,
      :instance_store_size     => 420.gigabytes,
      :instance_store_volumes  => 1,
      :architecture            => [:x86_64],
      :virtualization_type     => [:paravirtual],
      :network_performance     => :moderate,
      :physical_processor      => "Intel Xeon Family",
      :processor_clock_speed   => nil,
      :intel_aes_ni            => true,
      :intel_avx               => nil,
      :intel_avx2              => nil,
      :intel_turbo             => nil,
      :ebs_optimized_available => nil,
      :enhanced_networking     => nil,
      :cluster_networking      => nil,
      :vpc_only                => false,
    },

    "m2.2xlarge"  => {
      :name                    => "m2.2xlarge",
      :family                  => "Memory Optimized",
      :description             => "M2 High-Memory Double Extra Large",
      :memory                  => 34.2.gigabytes,
      :vcpu                    => 4,
      :ebs_only                => false,
      :instance_store_size     => 850.gigabytes,
      :instance_store_volumes  => 1,
      :architecture            => [:x86_64],
      :virtualization_type     => [:paravirtual],
      :network_performance     => :moderate,
      :physical_processor      => "Intel Xeon Family",
      :processor_clock_speed   => nil,
      :intel_aes_ni            => true,
      :intel_avx               => nil,
      :intel_avx2              => nil,
      :intel_turbo             => nil,
      :ebs_optimized_available => true,
      :enhanced_networking     => nil,
      :cluster_networking      => nil,
      :vpc_only                => false,
    },

    "m2.4xlarge"  => {
      :name                    => "m2.4xlarge",
      :family                  => "Memory Optimized",
      :description             => "M2 High-Memory Quadruple Extra Large",
      :memory                  => 68.4.gigabytes,
      :vcpu                    => 8,
      :ebs_only                => false,
      :instance_store_size     => 1680.gigabytes,
      :instance_store_volumes  => 2,
      :architecture            => [:x86_64],
      :virtualization_type     => [:paravirtual],
      :network_performance     => :high,
      :physical_processor      => "Intel Xeon Family",
      :processor_clock_speed   => nil,
      :intel_aes_ni            => true,
      :intel_avx               => nil,
      :intel_avx2              => nil,
      :intel_turbo             => nil,
      :ebs_optimized_available => true,
      :enhanced_networking     => nil,
      :cluster_networking      => nil,
      :vpc_only                => false,
    },

    "cr1.8xlarge" => {
      :name                    => "cr1.8xlarge",
      :family                  => "Memory Optimized",
      :description             => "High-Memory Cluster Eight Extra Large",
      :memory                  => 244.gigabytes,
      :vcpu                    => 32,
      :ebs_only                => false,
      :instance_store_size     => 240.gigabytes,
      :instance_store_volumes  => 2, # SSD
      :architecture            => [:x86_64],
      :virtualization_type     => [:hvm],
      :network_performance     => :very_high,
      :physical_processor      => "Intel Xeon E5-2670",
      :processor_clock_speed   => nil,
      :intel_aes_ni            => true,
      :intel_avx               => true,
      :intel_avx2              => nil,
      :intel_turbo             => true,
      :ebs_optimized_available => nil,
      :enhanced_networking     => nil,
      :cluster_networking      => true,
      :vpc_only                => false,
    },

    "hi1.4xlarge" => {
      :name                    => "hi1.4xlarge",
      :family                  => "Storage Optimized",
      :description             => "High I/O Quadruple Extra Large",
      :memory                  => 60.5.gigabytes,
      :vcpu                    => 16,
      :ebs_only                => false,
      :instance_store_size     => 2.terabytes,
      :instance_store_volumes  => 2, # SSD
      :architecture            => [:x86_64],
      :virtualization_type     => [:paravirtual, :hvm],
      :network_performance     => :very_high,
      :physical_processor      => "Intel Xeon Family",
      :processor_clock_speed   => 2.4, # GHz
      :intel_aes_ni            => true,
      :intel_avx               => nil,
      :intel_avx2              => nil,
      :intel_turbo             => nil,
      :ebs_optimized_available => nil,
      :enhanced_networking     => nil,
      :cluster_networking      => nil,
      :vpc_only                => false,
    },

    "hs1.8xlarge" => {
      :name                    => "hs1.8xlarge",
      :family                  => "Storage Optimized",
      :description             => "High Storage Eight Extra Large",
      :memory                  => 117.gigabytes,
      :vcpu                    => 16,
      :ebs_only                => false,
      :instance_store_size     => 48.terabytes,
      :instance_store_volumes  => 24,
      :architecture            => [:x86_64],
      :virtualization_type     => [:paravirtual, :hvm],
      :network_performance     => :very_high,
      :physical_processor      => "Intel Xeon E5-2650",
      :processor_clock_speed   => 2.0, # GHz
      :intel_aes_ni            => true,
      :intel_avx               => nil,
      :intel_avx2              => nil,
      :intel_turbo             => nil,
      :ebs_optimized_available => nil,
      :enhanced_networking     => nil,
      :cluster_networking      => true,
      :vpc_only                => false,
    },
  }

  # Types that are no longer advertised
  DISCONTINUED_TYPES = {
    "unknown"     => {
      :disabled                => true,
      :name                    => "unknown",
      :family                  => "unknown",
      :description             => "unknown",
      :memory                  => 0.megabytes,
      :vcpu                    => 1,
      :ebs_only                => true,
      :instance_store_size     => 0,
      :instance_store_volumes  => 0,
      :architecture            => [],
      :virtualization_type     => [],
      :network_performance     => :low_to_moderate,
      :physical_processor      => nil,
      :processor_clock_speed   => nil,
      :intel_aes_ni            => nil,
      :intel_avx               => nil,
      :intel_avx2              => nil,
      :intel_turbo             => nil,
      :ebs_optimized_available => nil,
      :enhanced_networking     => nil,
      :cluster_networking      => nil,
      :vpc_only                => false,
    },

    # cc1.4xlarge is not available in any of the documentation, but if you
    #   launch a new instance, it's still available to choose.
    "cc1.4xlarge" => {
      :name                    => "cc1.4xlarge",
      :family                  => "Compute Optimized",
      :description             => "Cluster Compute Quadruple Extra Large",
      :memory                  => 23.gigabytes,
      :vcpu                    => 16,
      :ebs_only                => false,
      :instance_store_size     => 1680.gigabytes,
      :instance_store_volumes  => 2,
      :architecture            => [:x86_64],
      :virtualization_type     => [:hvm],
      :network_performance     => :very_high,
      :physical_processor      => nil,
      :processor_clock_speed   => nil,
      :intel_aes_ni            => nil,
      :intel_avx               => nil,
      :intel_avx2              => nil,
      :intel_turbo             => nil,
      :ebs_optimized_available => nil,
      :enhanced_networking     => nil,
      :cluster_networking      => nil,
      :vpc_only                => false,
    },
  }

  def self.all
    AVAILABLE_TYPES.values + DEPRECATED_TYPES.values + DISCONTINUED_TYPES.values
  end

  def self.names
    AVAILABLE_TYPES.keys + DEPRECATED_TYPES.keys + DISCONTINUED_TYPES.keys
  end
end
